package org.integratedmodelling.klab.api.runtime;

import java.util.Collection;
import java.util.List;

/**
 * Simple, persistent ticket manager. Used to coordinate multi-actor operations
 * such as resource publishing. A ticket manager is exposed by the runtime
 * system in the engine and is used by nodes (and probably hubs).
 * 
 * @author Ferd
 *
 */
public interface ITicketManager {

	/**
	 * Open a new ticket. Arguments will be put in the ticket according to their
	 * type.
	 * 
	 * @param ticket
	 */
	ITicket open(Object... data);

	/**
	 * Return the ticket(s) correspondent to the passed selectors, which can be
	 * string IDs or any value(s) of the enums that describe a ticket.
	 * 
	 * @param selectors
	 * @return
	 */
	Collection<ITicket> get(Object... selectors);

	/**
	 * Return all tickets in order of creation.
	 * 
	 * @return
	 */
	List<ITicket> getTickets();

	/**
	 * Get a specific ticket, or null.
	 * 
	 * @param id
	 * @return
	 */
	ITicket getTicket(String id);

	/**
	 * Remove all tickets matching selectors
	 */
	void remove(Object... selectors);

	/**
	 * Get all tickets resolved after the passed time.
	 * 
	 * @param l
	 * @return
	 */
	Collection<ITicket> getResolvedAfter(long l);

}
